#ifndef SOLVER_H_
#define SOLVER_H_

#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include "matrix.h"
#include "vec.h"

// Problmes :
#define P_DECAY 0 // Les solutions du problme sont des dcroissances exponentielles
#define P_CIRCLE 1 // Les solutions du problme sont des cercles centrs sur l'origine
#define P_DYNAMICS 2 //dynamique
// Mthodes de rsolution :
#define M_EULER_FIX 10 // mthode d'Euler explicite  pas fixe
#define M_EULER_VAR 11 // mthode d'Euler explicite  pas variable
#define M_EULER_IMP 12 // mthode d'Euler implicite

class Solver {
public :
	double _step; // pas de temps
	double _tol; // tolrance  l'erreur
	double _k; // lasticit pour le problme des dcroissances exponentielles
	int _method; // mthode de rsolution
	int _problem; // problme
	Vec pos; // position actuelle de la solution3
	Vec pos0; // position initiale

	// Pour Dynamics :
	Vec derivee; // acceleration pour calculer la vlocit
	// vlocit pour calculer la position


public :
	Solver(){
		///////////////////////////////////////
		///////// PARAMETERS TO TUNE : ////////
		///////////////////////////////////////
		_k = 18;
		_step = 0.1;
		_tol = 0.1;
		_problem = P_DECAY;
		_method = M_EULER_FIX;
		pos0.x =0;
		pos0.y =0.5;
		pos0.z =0;
		///////////////////////////////////////
		///////////////////////////////////////
		///////////////////////////////////////
		pos.x = pos0.x;
		pos.y = pos0.y;
		pos.z = pos0.z;
	};
	~Solver(){};

	// Modifiers :
	void setStep(float step) {_step = step;}
	void setMethod(int method) {_method = method;}
	void setProblem(int problem) {_problem = problem;}

	void resetPos();

	// Effectue un pas de rsolution :
	void callMethod();
	// Calcule f(x,t) :
	void callProblem(Vec x, Vec* r);
	// Calcule le jacobien :
	void setJacobien(Vec x, Matrix* J);

	// Problmes :
	void decayForce(Vec x, Vec* r);
	void J_decayForce(Vec x, Matrix* J); // jacobien
	void circularForce(Vec x, Vec* r);
	void J_circularForce(Vec x, Matrix* J); // jacobien
	void dynamicsForce(Vec x, Vec* r);
	
	// Pas de rsolution :
	void eulerExpFixStep();
	void eulerExpVarStep();
	void eulerImp();

	void printPos() {
		printf("%f %f %f\n", pos.x, pos.y, pos.z);
	}
};

#endif